@@ -385,7 +385,7 @@ sub _searchLdap
 
     $rc = $msg->code;
     if ($rc) {
-        $self->log(level => 'err', message => "search by $filter failed($rc)");
+        $self->log(level => 'err', message => "ldap search by $filter failed($rc)");
         if ($rc == LDAP_SERVER_DOWN) {
             $ldapopts->{ldap}->unbind();
             undef($ldapopts->{ldap});
@@ -412,6 +412,53 @@ sub _searchLdap
     return @values;
 }
 
+sub _searchLism
+{
+    my $self = shift;
+    my ($lismopts, $filter) = @_;
+    my $base = $lismopts->{base};
+    my $scope = 2;
+    my $attr;
+    my @values = ('');
+
+    $filter =~ s/^["']//;
+    $filter =~ s/["']$//;
+    if ($filter =~ /^\(?dn=/) {
+        ($base) = ($filter =~ /^\(?dn=(.*)\)?$/);
+        $scope = 0;
+        $filter = '(objectClass=*)';
+    }
+    if ($lismopts->{attr} eq 'parentrdn') {
+        $attr = 'dn';
+    } else {
+        $attr = $lismopts->{attr};
+    }
+
+    my ($rc, @entries) = $self->{lism}->search($base, $scope, 0, 0, 0, $filter, 0, $attr);
+    if ($rc) {
+        $self->log(level => 'err', message => "lism search by $filter failed($rc)");
+
+        return undef;
+    }
+
+    if (@entries) {
+        my $entryStr = $entries[0];
+        if ($lismopts->{attr} eq 'dn') {
+            @values = ($entryStr =~ /^[^\n]+/);
+        } elsif ($lismopts->{attr} eq 'parentrdn') {
+            @values = ($entryStr =~ /^[^,]+,([^,]+)/);
+        } else {
+            @values = ($entryStr =~ /^$lismopts->{attr}: (.*)$/mi);
+        }
+
+        if (!defined($values[0])) {
+            @values = ('');
+        }
+    }
+
+    return @values;
+}
+
 sub _unique
 {
     my $self = shift;
