@@ -2,7 +2,9 @@ package LISM::Handler::Rewrite;
 
 use strict;
 use base qw(LISM::Handler);
+use Net::LDAP::Filter;
 use Encode;
+use LISM::Storage;
 use Data::Dumper;
 
 =head1 NAME
@@ -36,7 +38,7 @@ sub pre_bind
             }
 
             my $substitution = $rule->{substitution};
-            $substitution =~ s/%0/${$binddnp}/;
+            $substitution = $self->_rewritePattern($substitution, '%0', ${$binddnp});
 
             my $str = $self->_rewriteParse($rule->{match}, $substitution, ${$binddnp});
             if (!$str) {
@@ -70,7 +72,7 @@ sub pre_compare
 
             my %rwcache;
             my $substitution = $rule->{substitution};
-            $substitution =~ s/%0/${$dnp}\n${$avaStrp}/;
+            $substitution = $self->_rewritePattern($substitution, '%0', "${$dnp}\n${$avaStrp}");
 
             my $str = $self->_rewriteParse($rule->{match}, $substitution, ${$dnp}, \%rwcache);
             if (!$str) {
@@ -112,7 +114,7 @@ sub pre_search
             my %rwcache;
             my $substitution = $rule->{substitution};
 
-            $substitution =~ s/%0/${$basep}\n${$filterStrp}/;
+            $substitution = $self->_rewritePattern($substitution, '%0', "${$basep}\n${$filterStrp}");
             my $str = $self->_rewriteParse($rule->{match}, $substitution, ${$basep}, \%rwcache);
             if (!defined($str)) {
                 $self->log(level => 'err', message => "search rewrite \"${$basep}\" failed");
@@ -168,10 +170,13 @@ sub post_search
                 if (defined($rule->{dn}) && $dn !~ /$rule->{dn}/i) {
                     next;
                 }
+                if (defined($rule->{filter}) && !LISM::Storage->parseFilter($rule->{filterobj}, $entryStr)) {
+                    next;
+                }
 
                 my $rc = 0;
                 my $substitution = $rule->{substitution};
-                $substitution =~ s/%0/$entryStr/;
+                $substitution = $self->_rewritePattern($substitution, '%0', $entryStr);
 
                 my $str = $self->_rewriteParse($rule->{match}, $substitution, $dn, \%rwcache);
                 if (!$str) {
@@ -224,7 +229,7 @@ sub pre_modify
             my %rwcache;
             my $substitution = $rule->{substitution};
             my $modlist = "${$dnp}\n".join("\n", @{$listp});
-            $substitution =~ s/%0/$modlist/;
+            $substitution = $self->_rewritePattern($substitution, '%0', $modlist);
 
             my $str = $self->_rewriteParse($rule->{match}, $substitution, ${$dnp}, \%rwcache);
             if (!$str) {
@@ -295,10 +300,13 @@ sub pre_add
             if (defined($rule->{dn}) && ${$dnp} !~ /$rule->{dn}/i) {
                 next;
             }
+            if (defined($rule->{filter}) && !LISM::Storage->parseFilter($rule->{filterobj}, "${$dnp}\n${$entryStrp}")) {
+                next;
+            }
 
             my %rwcache;
             my $substitution = $rule->{substitution};
-            $substitution =~ s/%0/${$dnp}\n${$entryStrp}[0]/;
+            $substitution = $self->_rewritePattern($substitution, '%0', "${$dnp}\n${$entryStrp}[0]");
 
             my $str = $self->_rewriteParse($rule->{match}, $substitution, ${$dnp}, \%rwcache);
             if (!$str) {
@@ -343,7 +351,7 @@ sub pre_modrdn
 
             my %rwcache;
             my $substitution = $rule->{substitution};
-            $substitution =~ s/%0/${$dnp}\n${$argsp}[0]/;
+            $substitution = $self->_rewritePattern($substitution, '%0', "${$dnp}\n${$argsp}[0]");
 
             my $str = $self->_rewriteParse($rule->{match}, $substitution, ${$dnp}, \%rwcache);
             if (!$str) {
@@ -386,7 +394,7 @@ sub pre_delete
 
             my %rwcache;
             my $substitution = $rule->{substitution};
-            $substitution =~ s/%0/${$dnp}/;
+            $substitution = $self->_rewritePattern($substitution, '%0', ${$dnp});
 
             my $str = $self->_rewriteParse($rule->{match}, $substitution, ${$dnp}, \%rwcache);
             if (!$str) {
@@ -423,17 +431,27 @@ sub _checkConfig
 
     if (defined($conf->{rewritemap})) {
         foreach my $map_name (keys %{$conf->{rewritemap}}) {
-            if ($conf->{rewritemap}{$map_name}->{type} eq 'ldap') {
+            my $type = $conf->{rewritemap}{$map_name}->{type};
+            if ($type eq 'ldap') {
                 if (!defined($self->{ldapmap})) {$self->{ldapmap} = {}};
                 my $ldapmap = {};
                 $self->_parseLdapUri($ldapmap, $conf->{rewritemap}{$map_name}->{attrs});
                 $self->{ldapmap}{$map_name} = $ldapmap;
+            } elsif ($type eq 'lism') {
+                if (!defined($self->{lismmap})) {$self->{lismmap} = {}};
+                my $lismmap = {};
+                ($lismmap->{base}, $lismmap->{attr}) = split(/\?/, $conf->{rewritemap}{$map_name}->{attrs});
+                $self->{lismmap}{$map_name} = $lismmap;
             }
         }
     }
 
     if (defined($conf->{rewrite})) {
         foreach my $rule (@{$conf->{rewrite}}) {
+            if (defined($rule->{filter})) {
+                $rule->{filterobj} = Net::LDAP::Filter->new($rule->{filter});
+            }
+
             # enable escape sequence
             $rule->{substitution} =~ s/([^\\])\\n/$1\n/g;
             $rule->{substitution} =~ s/([^\\])\\t/$1\t/g;
@@ -443,6 +461,20 @@ sub _checkConfig
     return $rc;
 }
 
+sub _rewritePattern
+{
+    my $self = shift;
+    my ($str, $pattern, $value) = @_;
+
+    my ($qt) = ($str =~ /(['"])$pattern/);
+    if ($str =~ /$qt$pattern$qt/) {
+        $value =~ s/$qt/\\$qt/g;
+    }
+
+    $str =~ s/$pattern/$value/;
+    return $str;
+}
+
 sub _rewriteParse
 {
     my $self = shift;
@@ -457,7 +489,9 @@ sub _rewriteParse
     # replace variables
     for (my $i = 0; $i < @matches; $i++) {
         my $num = $i + 1;
-        $substitution =~ s/%$num/$matches[$i]/g;
+        while ($substitution =~ /%$num/) {
+            $substitution = $self->_rewritePattern($substitution, "%$num", $matches[$i]);
+        }
     }
 
     foreach my $substline ($self->_splitSubst($substitution)) {
@@ -469,6 +503,7 @@ sub _rewriteParse
         foreach my $rwmap (@rwmaps) {
             my @values;
             my $key = lc($rwmap);
+
             if (defined(${$rwcache}{$key})) {
                 @values = @{${$rwcache}{$key}};
             } else {
@@ -530,7 +565,7 @@ sub _splitSubst
         my $str = substr($substitution, $prevpos, $pos - $prevpos);
         if (index($str, "%{", $oldpos - $prevpos) > 0) {
             if (index($str, "(", $oldpos - $prevpos) > 0) {
-                 $oldpos = index($substitution, ")", $oldpos);
+                 $oldpos = $self->_passArgs($substitution, $oldpos);
             }
             $oldpos = index($substitution, "}", $oldpos);
             next;
@@ -543,6 +578,47 @@ sub _splitSubst
     return @substs;
 }
 
+sub _passArgs
+{
+    my $self = shift;
+    my ($str, $oldpos) = @_;
+    my $pos = $oldpos;
+
+    $pos = index($str, "(", $pos);
+    if ($pos < 0) {
+        return $pos;
+    }
+    $pos;
+
+    my $leftstr = substr($str, $pos + 1);
+    my ($qtchar) = ($leftstr =~ /^ *(['"])/);
+
+    while ($qtchar) {
+        while (1) {
+            $pos = index($str, $qtchar, $pos);
+            my $tmppos = index($str, $qtchar, $pos);
+            if ($tmppos < 0) {
+                last;
+            }
+            $pos = $tmppos + 1;
+
+            if (substr($str, $tmppos - 1, 1) eq "\\") {
+                next;
+            }
+            last;
+        }
+        $leftstr = substr($str, $pos);
+        ($qtchar) = ($leftstr =~ / *(['"])/);
+    }
+
+    $pos = index($str, ")", $pos);
+    if ($pos < 0) {
+        $pos = $oldpos;
+    }
+
+    return $pos;
+}
+
 sub _rewriteMap
 {
     my $self = shift;
@@ -567,6 +643,15 @@ sub _ldapMap
     return $self->_searchLdap($ldapmap, $map_args);
 }
 
+sub _lismMap
+{
+    my $self = shift;
+    my ($map_name, $map_args) = @_;
+    my $lismmap = $self->{lismmap}{$map_name};
+
+    return $self->_searchLism($lismmap, $map_args);
+}
+
 sub _functionMap
 {
     my $self = shift;
