@@ -2,8 +2,6 @@ package LISM::Handler::Script;
 
 use strict;
 use base qw(LISM::Handler);
-use Sys::Syslog;
-use Sys::Syslog qw(:macros);
 use Data::Dumper;
 
 =head1 NAME
@@ -222,14 +220,24 @@ sub _do_modify
             $cmd =~ s/%r/$rdn_val/g;
             $cmd =~ s/%i/$modinfo/g;
 
-            $rc = system($cmd);
-            if ($rc) {
-                $self->log(level => 'error', message => "Script $script($dn) in modify failed");
-                if ($rc != 1) {
-                    return $rc;
+            if ($^O ne 'MSWin32') {
+                open(CMD, "$cmd; echo status=\$?|");
+                while (<CMD>) {
+                    $rc = $_;
+                }
+                ($rc) = ($rc =~ /status=([0-9]+)/);
+                close(CMD);
+
+                if ($rc) {
+                    $self->log(level => 'err', message => "Script $script($dn) in modify failed($rc)");
+                    if ($rc != 1) {
+                        return $rc;
+                    }
+                } else {
+                    $self->log(level => 'info', message => "Script $script($dn) in modify succeeded");
                 }
             } else {
-                $self->log(level => 'info', message => "Script $script($dn) in modify succeeded");
+                system($cmd);
             }
         }
     }
@@ -297,14 +305,24 @@ sub _do_add
             $cmd =~ s/%r/$rdn_val/g;
             $cmd =~ s/%i/$addinfo/g;
 
-            $rc = system($cmd);
-            if ($rc) {
-                $self->log(level => 'error', message => "Script $script($dn) in add failed");
-                if ($rc != 1) {
-                    return $rc;
+            if ($^O ne 'MSWin32') {
+                open(CMD, "$cmd; echo status=\$?|");
+                while (<CMD>) {
+                    $rc = $_;
+                }
+                ($rc) = ($rc =~ /status=([0-9]+)/);
+                close(CMD);
+
+                if ($rc) {
+                    $self->log(level => 'err', message => "Script $script($dn) in add failed");
+                    if ($rc != 1) {
+                        return $rc;
+                    }
+                } else {
+                    $self->log(level => 'info', message => "Script $script($dn) in add succeeded");
                 }
             } else {
-                $self->log(level => 'info', message => "Script $script($dn) in add succeeded");
+                system($cmd);
             }
         }
     }
@@ -354,14 +372,24 @@ sub _do_delete
                 $cmd =~ s/%r/$rdn_val/g;
                 $cmd =~ s/%i/$info/g;
 
-                $rc = system($cmd);
-                if ($rc) {
-                    $self->log(level => 'error', message => "Script $script($dn) in deleting failed");
-                    if ($rc != 1) {
-                        return $rc;
+                if ($^O ne 'MSWin32') {
+                    open(CMD, "$cmd; echo status=\$?|");
+                    while (<CMD>) {
+                        $rc = $_;
+                    }
+                    ($rc) = ($rc =~ /status=([0-9]+)/);
+                    close(CMD);
+
+                    if ($rc) {
+                        $self->log(level => 'err', message => "Script $script($dn) in deleting failed");
+                        if ($rc != 1) {
+                            return $rc;
+                        }
+                    } else {
+                        $self->log(level => 'info', message => "Script $script($dn) in deleting succeeded");
                     }
                 } else {
-                    $self->log(level => 'info', message => "Script $script($dn) in deleting succeeded");
+                    system($cmd);
                 }
             }
             next;
