@@ -521,6 +521,7 @@ sub log
     if (Encode::is_utf8($p{'message'})) {
         $p{'message'} = encode('utf8', $p{'message'});
     }
+    $p{'message'} = '['.ref($self).'] '.$p{'message'};
 
     if ($^O ne 'MSWin32') {
         openlog('LISM', 'pid', 'local4');
@@ -1293,6 +1294,22 @@ sub _getStaticValue
     return $value;
 }
 
+sub _containerParse
+{
+    my $self = shift;
+    my ($str, @conts) = @_;
+
+    my @nums = ($str =~ /\%c([0-9]+)/g);
+    foreach my $num (@nums) {
+        if (defined($conts[$#conts + 1 - $num])) {
+            $str =~ s/\%c$num/$conts[$#conts + 1 - $num]/;
+        }
+    }
+    $str =~ s/\%c/$conts[$#conts]/g;
+
+    return $str;
+}
+
 sub _funcParse
 {
     my $self = shift;
