@@ -77,6 +77,10 @@ sub rollback
     my $conf = $self->{_config};
 
     foreach my $oname (keys %{$conf->{object}}) {
+        if (!defined($conf->{object}{$oname}->{file})) {
+            next;
+        }
+
         my $file = $conf->{object}{$oname}->{file}[0];
         if (!open(LOCK, "> $file.lock")) {
             $self->log(level => 'alert', message => "Can't open $file.lock");
@@ -135,24 +139,8 @@ sub _objSearch
     my $rc = LDAP_SUCCESS;
 
     DO: {
-        my $lock;
-        my $file;
-
-        if (!$self->_checkFile($oconf->{file}[0])) {
-            $rc = LDAP_OPERATIONS_ERROR;
-            last DO;
-        }
-
-        if (!open($lock, "> $oconf->{file}[0].lock")) {
-            $self->log(level => 'alert', message => "Can't open $oconf->{file}[0].lock");
-            $rc = LDAP_OPERATIONS_ERROR;
-            last DO;
-        }
-        flock($lock, 1);
-
-        if (!open($file, "< $oconf->{file}[0]")) {
-            $self->log(level => 'alert', message => "Can't open $oconf->{file}[0]");
-            close($lock);
+        my ($file, $lock) = $self->_openRead($oconf);
+        if (!defined($file)) {
             $rc = LDAP_OPERATIONS_ERROR;
             last DO;
         }
@@ -161,6 +149,34 @@ sub _objSearch
             s/\r|\n|\r\n$//;
             my @data = split(/$dlm/);
             my $entry;
+            my $match = 1;
+
+            foreach my $strginfo (@{$oconf->{strginfo}}) {
+                if (defined($strginfo->{column}) && defined($strginfo->{filter}) && defined($strginfo->{filter}->{type})) {
+                    my $value = $data[$strginfo->{column}[0]];
+                    my $ftype = $strginfo->{filter}->{type};
+                    my $fval = $strginfo->{filter}->{content};
+                    if ($ftype eq 'eq') {
+                        $match = $value == $fval ? 1 : 0;
+                    } elsif ($ftype eq 'not') {
+                        $match = $value != $fval ? 1 : 0;
+                    } elsif ($ftype eq 'lt') {
+                        $match = $value < $fval ? 1 : 0;
+                    } elsif ($ftype eq 'gt') {
+                        $match = $value > $fval ? 1 : 0;
+                    } elsif ($ftype eq 'le') {
+                        $match = $value <= $fval ? 1 : 0;
+                    } elsif ($ftype eq 'ge') {
+                        $match = $value >= $fval ? 1 : 0;
+                    }
+                    if (!$match) {
+                        last;
+                    }
+                }
+            }
+            if (!$match) {
+                next;
+            }
 
             # check the number of returned entries
             if ($sizeLim >= 0 && @match_entries == $sizeLim) {
@@ -212,8 +228,9 @@ sub _objSearch
                 push(@match_keys, $data[$oconf->{id}[0]->{column}[0]]);
             }
         }
-        close($file);
-        close($lock);
+        if ($self->_close($file, $lock, $oconf)) {
+            $rc = LDAP_OPERATIONS_ERROR;
+        }
     }
 
     return ($rc , \@match_keys, @match_entries);
@@ -239,6 +256,10 @@ sub _objModify
     my $match = 0;
     my $rc = LDAP_SUCCESS;
 
+    if (!defined($oconf->{file})) {
+        return $rc;
+    }
+
     my ($rdn_val) = ($dn =~ /^[^=]+=([^,]+),/);
     $rdn_val =~ s/([.*+?\[\]()|\^\$\\])/\\$1/g;
 
@@ -426,6 +447,10 @@ sub _objAdd
     my $valdlm = $conf->{valdelim}[0];
     my $rc = LDAP_SUCCESS;
 
+    if (!defined($oconf->{file})) {
+        return $rc;
+    }
+
     my ($rdn_val) = ($dn =~ /^[^=]+=([^,]+),/);
     $rdn_val =~ s/([.*+?\[\]()|\^\$\\])/\\$1/g;
 
@@ -527,9 +552,8 @@ sub _objAdd
 
             # add storage-specific information
             foreach my $strginfo (@{$oconf->{strginfo}}) {
-                my $value = $self->_getStaticValue($strginfo, $dn, $entryStr);
-
-                if (defined($strginfo->{column})) {
+                if (defined($strginfo->{column}) && defined($strginfo->{value})) {
+                    my $value = $self->_getStaticValue($strginfo, $dn, $entryStr);
                     $data[$strginfo->{column}[0]] = $value;
                 }
             }
@@ -571,6 +595,10 @@ sub _objDelete
     my $valdlm = $conf->{valdelim}[0];
     my $rc = LDAP_NO_SUCH_OBJECT;
 
+    if (!defined($oconf->{file})) {
+        return $rc;
+    }
+
     my ($rdn_val) = ($dn =~ /^[^=]+=([^,]+),/);
     $rdn_val =~ s/([.*+?\[\]()|\^\$\\])/\\$1/g;
 
@@ -660,24 +688,11 @@ sub _getParentRdn
         return undef;
     }
 
-    my $lock;
-    my $file;
-
-    if (!$self->_checkFile($oconf->{file}[0])) {
-        return LDAP_OPERATIONS_ERROR;
+    my ($file, $lock) = $self->_openRead($oconf);
+    if (!defined($file)) {
+        return undef;
     }
 
-    if (!open($lock, "> $oconf->{file}[0].lock")) {
-        $self->log(level => 'alert', message => "Can't open $oconf->{file}[0].lock");
-        return LDAP_OPERATIONS_ERROR;
-    }
-    flock($lock, 1);
-
-    if (!open($file, "< $oconf->{file}[0]")) {
-        $self->log(level => 'alert', message => "Can't open $oconf->{file}[0]");
-        close($lock);
-        return LDAP_OPERATIONS_ERROR;
-    }
     while (<$file>) {
         chop;
         my @data = split(/$dlm/);
@@ -689,20 +704,13 @@ sub _getParentRdn
         }
     }
 
-    my $poconf = $self->{object}{$obj->{parent}}->{conf};
-    if (!$self->_checkFile($poconf->{file}[0])) {
-        return LDAP_OPERATIONS_ERROR;
-    }
-
-    if (!open($lock, "> $poconf->{file}[0].lock")) {
-        $self->log(level => 'alert', message => "Can't open $poconf->{file}[0].lock");
+    if ($self->_close($file, $lock, $oconf)) {
         return undef;
     }
-    flock($lock, 1);
 
-    if (!open($file, "< $poconf->{file}[0]")) {
-        $self->log(level => 'alert', message => "Can't open $poconf->{file}[0]");
-        close($lock);
+    my $poconf = $self->{object}{$obj->{parent}}->{conf};
+    ($file, $lock) = $self->_openRead($poconf);
+    if (!defined($file)) {
         return undef;
     }
 
@@ -718,8 +726,9 @@ sub _getParentRdn
             last;
         }
     }
-    close($lock);
-    close($file);
+    if ($self->_close($file, $lock, $poconf)) {
+        return undef;
+    }
 
     return ($prdn, $pkey);
 }
@@ -736,33 +745,20 @@ sub _getAttrValues
     my @colnums;
     my $filename;
 
+    my $file;
+    my $lock;
     if (defined($oconf->{attr}{$attr}->{oname})) {
         $aobj = $self->{object}{$oconf->{attr}{$attr}->{oname}[0]};
         $aoconf = $aobj->{conf};
-        $filename = $aoconf->{file}[0];
+        ($file, $lock) = $self->_openRead($aoconf);
     } elsif (defined($oconf->{attr}{$attr}->{file})) {
         @colnums = ($oconf->{attr}{$attr}->{value}[0] =~ /%([0-9]+)/g);
-        $filename = $oconf->{attr}{$attr}->{file}[0];
+        ($file, $lock) = $self->_openRead($oconf->{attr}{$attr});
     } else {
         return undef;
     }
 
-    my $lock;
-    my $file;
-
-    if (!$self->_checkFile($filename)) {
-        return LDAP_OPERATIONS_ERROR;
-    }
-
-    if (!open($lock, "> $filename.lock")) {
-        $self->log(level => 'alert', message => "Can't open $filename.lock");
-        return undef;
-    }
-    flock($lock, 1);
-
-    if (!open($file, "< $filename")) {
-        $self->log(level => 'alert', message => "Can't open $filename");
-        close($lock);
+    if (!defined($file)) {
         return undef;
     }
 
@@ -801,8 +797,10 @@ sub _getAttrValues
             last;
         }
     }
-    close($lock);
-    close($file);
+
+    if ($self->_close($file, $lock, $oconf->{attr}{$attr})) {
+        return undef;
+    }
 
     # Values not got exist
     if (@keys) {
@@ -840,25 +838,11 @@ sub _getAttrKeys
             push(@attrkeys, $attrkey);
         }
     } elsif (defined($oconf->{attr}{$attr}->{file})) {
-        my $lock;
-        my $file;
-
-        if (!$self->_checkFile($oconf->{attr}{$attr}->{file}[0])) {
+        my ($file, $lock) = $self->_openRead($oconf->{attr}{$attr});
+        if (!defined($file)) {
             return LDAP_OPERATIONS_ERROR;
         }
 
-        if (!open($lock, "> $oconf->{attr}{$attr}->{file}[0].lock")) {
-            $self->log(level => 'alert', message => "Can't open $oconf->{attr}{$attr}->{file}[0].lock");
-            return (-1, ());
-        }
-        flock($lock, 1);
-
-        if (!open($file, "< $oconf->{attr}{$attr}->{file}[0]")) {
-            $self->log(level => 'alert', message => "Can't open $oconf->{attr}{$attr}->{file}[0]");
-            close($lock);
-            return (-1, ());
-        }
-
         my @colnums = ($oconf->{attr}{$attr}->{value}[0] =~ /%([0-9]+)/g);
         (my $replace = $oconf->{attr}{$attr}->{value}[0]) =~ s/([*+\/\.^$()\[\]])/\\$1/g;
         $replace =~ s/%[0-9]+/(.+)/ig;
@@ -894,8 +878,10 @@ sub _getAttrKeys
 		}
 	    }
         }
-        close($lock);
-        close($file);
+
+        if ($self->_close($file, $lock, $oconf->{attr}{$attr})) {
+            return (-1, ());
+        }
 
         # Values not added exist
         if (@avals) {
@@ -926,6 +912,57 @@ sub _checkFile
     return 1;
 }
 
+sub _openRead
+{
+    my $self = shift;
+    my ($oconf) = @_;
+    my $file;
+    my $lock;
+
+    if (defined($oconf->{file})) {
+        if (!$self->_checkFile($oconf->{file}[0])) {
+            return undef;
+        }
+
+        if (!open($lock, "> $oconf->{file}[0].lock")) {
+            $self->log(level => 'alert', message => "Can't open $oconf->{file}[0].lock");
+            return undef;
+        }
+        flock($lock, 1);
+
+        if (!open($file, "< $oconf->{file}[0]")) {
+            $self->log(level => 'alert', message => "Can't open $oconf->{file}[0]");
+            close($lock);
+            return undef;
+        }
+    } elsif (defined($oconf->{command})) {
+        if (!open($file, "$oconf->{command}[0]|")) {
+            $self->log(level => 'alert', message => "Can't open \"$oconf->{command}[0]\"");
+            return undef;
+        }
+    }
+
+    return ($file, $lock);
+}
+
+sub _close
+{
+    my $self = shift;
+    my ($file, $lock, $oconf) = @_;
+    my $rc = 0;
+
+    close($file);
+    if (defined($oconf->{command}) && $? != 0) {
+        $rc = -1;
+    }
+
+    if ($lock) {
+        close($lock);
+    }
+
+    return $rc;
+}
+
 =head1 SEE ALSO
 
 L<LISM>,
