@@ -273,7 +273,7 @@ sub _objSearch
 
             if (defined($oconf->{container}[0]->{joinwhere})) {
                 my $pwhere = $oconf->{container}[0]->{joinwhere}[0];
-                $pwhere =~ s/\%c/$pkey/g;
+                $pwhere = $self->_containerParse($pwhere, @{$pkeys});
                 $where = "$where and $pwhere";
             }
         }
@@ -776,7 +776,7 @@ sub _objAdd
         if (defined($oconf->{container}) && defined($oconf->{container}[0]->{addproc})) {
             $sql = $oconf->{container}[0]->{addproc}[0];
             $sql =~ s/\%o/$key/g;
-            $sql =~ s/\%c/$pkey/g;
+            $sql = $self->_containerParse($sql, @{$pkeys});
             $sql = $self->_funcParse($sql, $dn, $entryStr);
             $sth = $self->{db}->prepare("$sql");
             if (!$sth->execute) {
@@ -877,7 +877,7 @@ sub _objDelete
             if (defined($oconf->{container}) && defined($oconf->{container}[0]->{delproc})) {
                 $sql = $oconf->{container}[0]->{delproc}[0];
                 $sql =~ s/\%o/$key/g;
-                $sql =~ s/\%c/$pkey/g;
+                $sql = $self->_containerParse($sql, @{$pkeys});
                 $sql = $self->_funcParse($sql, $dn);
                 $sth = $self->{db}->prepare("$sql");
                 if (!$sth->execute) {
