@@ -8,6 +8,7 @@ BEGIN {
   undef $LISM_SERVER unless $LISM_SERVER and -x $LISM_SERVER;
   undef $SQL_SERVER unless $SQL_SERVER and -x $SQL_SERVER;
   undef $LDAP_SERVER unless $LDAP_SERVER and -x $LDAP_SERVER;
+  undef $HTTP_SERVER unless $HTTP_SERVER and -x $HTTP_SERVER;
 
   # If your host cannot be contacted as localhost, change this
   $HOST     ||= '127.0.0.1';
@@ -18,6 +19,9 @@ BEGIN {
   $WD = Cwd::getcwd();
   $TEMPDIR  = "$WD/temp";
   $DATADIR  = "$WD/data";
+  if (!$TRANSACTION) {
+    $TRANSACTION = 'off';
+  }
 
   $TESTDB   = "$TEMPDIR/test-db";
   $CONF     = "$TEMPDIR/conf";
@@ -63,6 +67,11 @@ BEGIN {
   $TESTCSV  = "$TEMPDIR/test-csv";
   $CSVDN    = "ou=CSV,dc=lism,dc=com";
 
+  $SOAPCONF = "$TEMPDIR/httpd.conf";
+  $SOAPDN   = "ou=SOAP,dc=lism,dc=com";
+  $SOAPCONF_IN    = "$DATADIR/httpd-conf.in";
+  @HTTPD          = ($HTTP_SERVER, '-f',$SOAPCONF);
+
   mkdir($TEMPDIR,0777);
   die "$TEMPDIR is not a directory" unless -d $TEMPDIR;
 }
@@ -77,6 +86,7 @@ my $pid;
 my $sqlpid;
 my $ldappid;
 my $slavepid;
+my $httpdpid;
 
 sub start_server {
   my %arg = (version => 2, @_);
@@ -148,6 +158,18 @@ sub start_server {
     close(SLAVECONFO);
   }
 
+  if ($SOAPCONF_IN and -f $SOAPCONF_IN) {
+    # Create slapd config file
+    open(SOAPCONFI,"<$SOAPCONF_IN") or die "$!";
+    open(SOAPCONFO,">$SOAPCONF") or die "$!";
+    while(<SOAPCONFI>) {
+      s/\$([A-Z]\w*)/${$1}/g;
+      print SOAPCONFO;
+    }
+    close(SOAPCONFI);
+    close(SOAPCONFO);
+  }
+
   rmtree($TESTDB) if ( -d $TESTDB );
   mkdir($TESTDB,0777);
   die "$TESTDB is not a directory" unless -d $TESTDB;
@@ -228,6 +250,17 @@ sub start_server {
 
   system($LDAPADD, @SLAVE_OPTS, '-f', "$DATADIR/initldap.ldif");
 
+  unlink("$TEMPDIR/access_log");
+  unlink("$TEMPDIR/error_log");
+
+  unless ($httpdpid = fork) {
+    die "fork: $!" unless defined $httpdpid;
+
+    exec(@HTTPD) or die "cannot exec @HTTPD";
+  }
+
+  sleep 2;
+
   warn "@LISMD" if $ENV{TEST_VERBOSE};
 
   $log = $TEMPDIR . "/" . basename($0,'.t');
@@ -242,7 +275,7 @@ sub start_server {
     exec(@LISMD) or die "cannot exec @LISMD";
   }
 
-  sleep 4;
+  sleep 6;
 }
 
 sub kill_server {
@@ -269,6 +302,18 @@ sub kill_server {
     sleep 1;
     undef $slavepid;
   }
+
+  if ($httpdpid) {
+    open(PIDF, "< $TEMPDIR/httpd.pid");
+    while (<PIDF>) {
+      chop;
+      kill 15, $_;
+      last;
+    }
+    close(PIDF);
+    sleep 1;
+    undef $httpdpid;
+  }
 }
 
 END {
